package org.dive4elements.river.artifacts.model.minfo;

import org.apache.log4j.Logger;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.FLYSArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.artifacts.context.FLYSContext;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.math.MovingAverage;
import org.dive4elements.river.artifacts.model.ZoomScale;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;


public class BedDiffYearFilterFacet
extends DataFacet
{
    private static Logger logger = Logger.getLogger(BedDiffYearFilterFacet.class);

    public BedDiffYearFilterFacet() {
    }

    public BedDiffYearFilterFacet(int idx, String name, String description,
        ComputeType type, String stateId, String hash) {
        super(idx, name, description, type, hash, stateId);
    }

    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("Get data for bed density at index: " + index);

        FLYSArtifact flys = (FLYSArtifact) artifact;

        CalculationResult res = (CalculationResult) flys.compute(context, hash,
            stateId, type, false);

        BedDiffYearResult[] data =
            (BedDiffYearResult[]) res.getData(); // TODO CAST TO SPECIFIC CLASS
        Double start = (Double)context.getContextValue("startkm");
        Double end = (Double)context.getContextValue("endkm");
        if(start != null && end != null) {
            FLYSContext fc = (FLYSContext)context.globalContext();
            ZoomScale scales = (ZoomScale)fc.get("zoomscale");
            RiverAccess access = new RiverAccess((FLYSArtifact)artifact);
            String river = access.getRiver();

            double radius = scales.getRadius(river, start, end);
            BedDiffYearResult oldData = data[index];
            BedDiffYearResult newData = new BedDiffYearResult();
            newData.setStart(oldData.getStart());
            newData.setEnd(oldData.getEnd());
            double[][] diffs = MovingAverage.weighted(oldData.getDifferencesData(), radius);
            double[][] heights1 = MovingAverage.weighted(oldData.getHeights1Data(), radius);
            double[][] heights2 = MovingAverage.weighted(oldData.getHeights2Data(), radius);
            double[][] morph = oldData.getMorphWidthData();
            double[][] year = MovingAverage.weighted(oldData.getHeightPerYearData(), radius);
            for(int j = 0; j < diffs[0].length; j++) {
                newData.addKm(diffs[0][j]);
                newData.addBedHeights(year[1][j]);
                newData.addMorphWidth(morph[1][j]);
                newData.addDifference(diffs[1][j]);
                newData.addHeight1(heights1[1][j]);
                newData.addHeight2(heights2[1][j]);
            }
            return newData;
        }
        return data != null && data.length > index ? data[index] : null;
    }

    /** Copy deeply. */
    @Override
    public Facet deepCopy() {
        BedDiffYearFilterFacet copy = new BedDiffYearFilterFacet();
        copy.set(this);
        copy.type = type;
        copy.hash = hash;
        copy.stateId = stateId;
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
